@@ -1,9 +1,9 @@
+from google.cloud.logging import Client
 from pydantic import BaseModel, ConfigDict, Field, model_validator
 from typing import Optional, Self
-from maleo.types.base.dict import OptionalStringToStringDict
-from maleo.types.base.string import OptionalString
+from maleo.types.dict import OptionalStringToStringDict
+from maleo.types.string import OptionalString
 from .enums import Level
-from .google import GoogleCloudLogging
 
 
 class Config(BaseModel):
@@ -11,8 +11,8 @@ class Config(BaseModel):
 
     dir: str = Field(..., description="Log's directory")
     level: Level = Field(Level.INFO, description="Log's level")
-    google_cloud_logging: Optional[GoogleCloudLogging] = Field(
-        default_factory=GoogleCloudLogging, description="Google cloud logging"
+    google_cloud_logging: Optional[Client] = Field(
+        None, description="Google cloud logging"
     )
     labels: OptionalStringToStringDict = Field(
         None, description="Log labels. (Optional)"
