@@ -1,25 +1,35 @@
 import logging
 import os
 from datetime import datetime, timezone
+from enum import StrEnum
+from google.cloud.logging.handlers import CloudLoggingHandler
 from pydantic import BaseModel, ConfigDict, Field
-from typing import Literal, Optional, Union, overload
-from maleo.enums.environment import Environment
-from maleo.enums.service import Key
-from maleo.types.base.dict import StringToStringDict
-from maleo.types.base.string import OptionalString
+from typing import Generic, Literal, Optional, TypeVar, Union, overload
+from maleo.types.dict import StringToStringDict
+from maleo.types.string import OptionalString
 from .config import Config
 from .enums import LoggerType
 
 
+EnvironmentT = TypeVar("EnvironmentT", str, StrEnum)
+ServiceKeyT = TypeVar("ServiceKeyT", str, StrEnum)
+
+
 # * We suggest to NOT use this class
 # * Instead use the inherited classes
-class Base(logging.Logger):
+class Base(
+    logging.Logger,
+    Generic[
+        EnvironmentT,
+        ServiceKeyT,
+    ],
+):
     def __init__(
         self,
         type: LoggerType = LoggerType.BASE,
         *,
-        environment: Optional[Environment] = None,
-        service_key: Optional[Key] = None,
+        environment: Optional[EnvironmentT] = None,
+        service_key: Optional[ServiceKeyT] = None,
         client_key: OptionalString = None,
         config: Config,
     ):
@@ -32,7 +42,7 @@ class Base(logging.Logger):
                 "ENVIRONMENT environment variable must be set if 'environment' is set to None"
             )
         else:
-            self._environment = Environment(actual_environment)
+            self._environment = actual_environment
 
         # Ensure service_key exists
         actual_service_key = service_key or os.getenv("SERVICE_KEY")
@@ -41,7 +51,7 @@ class Base(logging.Logger):
                 "SERVICE_KEY environment variable must be set if 'service_key' is set to None"
             )
         else:
-            self._service_key = Key(actual_service_key)
+            self._service_key = actual_service_key
 
         self._client_key = client_key  # Declare client key
 
@@ -61,8 +71,8 @@ class Base(logging.Logger):
         # Define log labels
         self._labels: StringToStringDict = {
             "logger_type": self._type.value,
-            "service_environment": self._environment.value,
-            "service_key": self._service_key.value,
+            "service_environment": str(self._environment),
+            "service_key": str(self._service_key),
         }
         if client_key is not None:
             self._labels["client_key"] = client_key
@@ -87,11 +97,12 @@ class Base(logging.Logger):
 
         # Google Cloud Logging handler (If enabled)
         if self._config.google_cloud_logging is not None:
-            cloud_logging_handler = self._config.google_cloud_logging.create_handler(
+            self._cloud_logging_handler = CloudLoggingHandler(
+                self._config.google_cloud_logging,
                 name=self._name.replace(" ", ""),
                 labels=self._labels,
             )
-            self.addHandler(cloud_logging_handler)
+            self.addHandler(self._cloud_logging_handler)
         else:
             self.warning(
                 "Cloud logging is not configured. Will not add cloud logging handler"
@@ -154,12 +165,12 @@ class Base(logging.Logger):
         self.handlers.clear()
 
 
-class Application(Base):
+class Application(Base[EnvironmentT, ServiceKeyT], Generic[EnvironmentT, ServiceKeyT]):
     def __init__(
         self,
         *,
-        environment: Optional[Environment] = None,
-        service_key: Optional[Key] = None,
+        environment: Optional[EnvironmentT] = None,
+        service_key: Optional[ServiceKeyT] = None,
         config: Config,
     ):
         super().__init__(
@@ -171,12 +182,12 @@ class Application(Base):
         )
 
 
-class Cache(Base):
+class Cache(Base[EnvironmentT, ServiceKeyT], Generic[EnvironmentT, ServiceKeyT]):
     def __init__(
         self,
         *,
-        environment: Optional[Environment] = None,
-        service_key: Optional[Key] = None,
+        environment: Optional[EnvironmentT] = None,
+        service_key: Optional[ServiceKeyT] = None,
         config: Config,
     ):
         super().__init__(
@@ -188,12 +199,12 @@ class Cache(Base):
         )
 
 
-class Client(Base):
+class Client(Base[EnvironmentT, ServiceKeyT], Generic[EnvironmentT, ServiceKeyT]):
     def __init__(
         self,
         *,
-        environment: Optional[Environment] = None,
-        service_key: Optional[Key] = None,
+        environment: Optional[EnvironmentT] = None,
+        service_key: Optional[ServiceKeyT] = None,
         client_key: str,
         config: Config,
     ):
@@ -206,12 +217,12 @@ class Client(Base):
         )
 
 
-class Controller(Base):
+class Controller(Base[EnvironmentT, ServiceKeyT], Generic[EnvironmentT, ServiceKeyT]):
     def __init__(
         self,
         *,
-        environment: Optional[Environment] = None,
-        service_key: Optional[Key] = None,
+        environment: Optional[EnvironmentT] = None,
+        service_key: Optional[ServiceKeyT] = None,
         config: Config,
     ):
         super().__init__(
@@ -223,12 +234,12 @@ class Controller(Base):
         )
 
 
-class Database(Base):
+class Database(Base[EnvironmentT, ServiceKeyT], Generic[EnvironmentT, ServiceKeyT]):
     def __init__(
         self,
         *,
-        environment: Optional[Environment] = None,
-        service_key: Optional[Key] = None,
+        environment: Optional[EnvironmentT] = None,
+        service_key: Optional[ServiceKeyT] = None,
         config: Config,
     ):
         super().__init__(
@@ -240,12 +251,12 @@ class Database(Base):
         )
 
 
-class Exception(Base):
+class Exception(Base[EnvironmentT, ServiceKeyT], Generic[EnvironmentT, ServiceKeyT]):
     def __init__(
         self,
         *,
-        environment: Optional[Environment] = None,
-        service_key: Optional[Key] = None,
+        environment: Optional[EnvironmentT] = None,
+        service_key: Optional[ServiceKeyT] = None,
         config: Config,
     ):
         super().__init__(
@@ -257,12 +268,12 @@ class Exception(Base):
         )
 
 
-class Middleware(Base):
+class Middleware(Base[EnvironmentT, ServiceKeyT], Generic[EnvironmentT, ServiceKeyT]):
     def __init__(
         self,
         *,
-        environment: Optional[Environment] = None,
-        service_key: Optional[Key] = None,
+        environment: Optional[EnvironmentT] = None,
+        service_key: Optional[ServiceKeyT] = None,
         config: Config,
     ):
         super().__init__(
@@ -274,12 +285,12 @@ class Middleware(Base):
         )
 
 
-class Repository(Base):
+class Repository(Base[EnvironmentT, ServiceKeyT], Generic[EnvironmentT, ServiceKeyT]):
     def __init__(
         self,
         *,
-        environment: Optional[Environment] = None,
-        service_key: Optional[Key] = None,
+        environment: Optional[EnvironmentT] = None,
+        service_key: Optional[ServiceKeyT] = None,
         config: Config,
     ):
         super().__init__(
@@ -291,12 +302,12 @@ class Repository(Base):
         )
 
 
-class Service(Base):
+class Service(Base[EnvironmentT, ServiceKeyT], Generic[EnvironmentT, ServiceKeyT]):
     def __init__(
         self,
         *,
-        environment: Optional[Environment] = None,
-        service_key: Optional[Key] = None,
+        environment: Optional[EnvironmentT] = None,
+        service_key: Optional[ServiceKeyT] = None,
         config: Config,
     ):
         super().__init__(
@@ -312,109 +323,109 @@ class Service(Base):
 def create(
     type: Literal[LoggerType.APPLICATION],
     *,
-    environment: Optional[Environment] = None,
-    service_key: Optional[Key] = None,
+    environment: Optional[EnvironmentT] = None,
+    service_key: Optional[ServiceKeyT] = None,
     config: Config,
-) -> Application: ...
+) -> Application[EnvironmentT, ServiceKeyT]: ...
 @overload
 def create(
     type: Literal[LoggerType.CACHE],
     *,
-    environment: Optional[Environment] = None,
-    service_key: Optional[Key] = None,
+    environment: Optional[EnvironmentT] = None,
+    service_key: Optional[ServiceKeyT] = None,
     config: Config,
-) -> Cache: ...
+) -> Cache[EnvironmentT, ServiceKeyT]: ...
 @overload
 def create(
     type: Literal[LoggerType.CLIENT],
     *,
-    environment: Optional[Environment] = None,
-    service_key: Optional[Key] = None,
+    environment: Optional[EnvironmentT] = None,
+    service_key: Optional[ServiceKeyT] = None,
     client_key: str,
     config: Config,
-) -> Client: ...
+) -> Client[EnvironmentT, ServiceKeyT]: ...
 @overload
 def create(
     type: Literal[LoggerType.CONTROLLER],
     *,
-    environment: Optional[Environment] = None,
-    service_key: Optional[Key] = None,
+    environment: Optional[EnvironmentT] = None,
+    service_key: Optional[ServiceKeyT] = None,
     config: Config,
-) -> Controller: ...
+) -> Controller[EnvironmentT, ServiceKeyT]: ...
 @overload
 def create(
     type: Literal[LoggerType.DATABASE],
     *,
-    environment: Optional[Environment] = None,
-    service_key: Optional[Key] = None,
+    environment: Optional[EnvironmentT] = None,
+    service_key: Optional[ServiceKeyT] = None,
     config: Config,
-) -> Database: ...
+) -> Database[EnvironmentT, ServiceKeyT]: ...
 @overload
 def create(
     type: Literal[LoggerType.EXCEPTION],
     *,
-    environment: Optional[Environment] = None,
-    service_key: Optional[Key] = None,
+    environment: Optional[EnvironmentT] = None,
+    service_key: Optional[ServiceKeyT] = None,
     config: Config,
-) -> Exception: ...
+) -> Exception[EnvironmentT, ServiceKeyT]: ...
 @overload
 def create(
     type: Literal[LoggerType.MIDDLEWARE],
     *,
-    environment: Optional[Environment] = None,
-    service_key: Optional[Key] = None,
+    environment: Optional[EnvironmentT] = None,
+    service_key: Optional[ServiceKeyT] = None,
     config: Config,
-) -> Middleware: ...
+) -> Middleware[EnvironmentT, ServiceKeyT]: ...
 @overload
 def create(
     type: Literal[LoggerType.REPOSITORY],
     *,
-    environment: Optional[Environment] = None,
-    service_key: Optional[Key] = None,
+    environment: Optional[EnvironmentT] = None,
+    service_key: Optional[ServiceKeyT] = None,
     config: Config,
-) -> Repository: ...
+) -> Repository[EnvironmentT, ServiceKeyT]: ...
 @overload
 def create(
     type: Literal[LoggerType.SERVICE],
     *,
-    environment: Optional[Environment] = None,
-    service_key: Optional[Key] = None,
+    environment: Optional[EnvironmentT] = None,
+    service_key: Optional[ServiceKeyT] = None,
     config: Config,
-) -> Service: ...
+) -> Service[EnvironmentT, ServiceKeyT]: ...
 def create(
     type: LoggerType = LoggerType.BASE,
     *,
-    environment: Optional[Environment] = None,
-    service_key: Optional[Key] = None,
+    environment: Optional[EnvironmentT] = None,
+    service_key: Optional[ServiceKeyT] = None,
     client_key: OptionalString = None,
     config: Config,
 ) -> Union[
-    Base,
-    Application,
-    Cache,
-    Client,
-    Controller,
-    Database,
-    Exception,
-    Middleware,
-    Repository,
-    Service,
+    Base[EnvironmentT, ServiceKeyT],
+    Application[EnvironmentT, ServiceKeyT],
+    Cache[EnvironmentT, ServiceKeyT],
+    Client[EnvironmentT, ServiceKeyT],
+    Controller[EnvironmentT, ServiceKeyT],
+    Database[EnvironmentT, ServiceKeyT],
+    Exception[EnvironmentT, ServiceKeyT],
+    Middleware[EnvironmentT, ServiceKeyT],
+    Repository[EnvironmentT, ServiceKeyT],
+    Service[EnvironmentT, ServiceKeyT],
 ]:
     if type is LoggerType.BASE:
-        return Base(
+        return Base[EnvironmentT, ServiceKeyT](
             environment=environment,
             service_key=service_key,
             client_key=client_key,
             config=config,
         )
     elif type is LoggerType.APPLICATION:
-        return Application(
+        return Application[EnvironmentT, ServiceKeyT](
             environment=environment,
             service_key=service_key,
             config=config,
         )
     elif type is LoggerType.CACHE:
-        return Cache(
+        return Cache[EnvironmentT, ServiceKeyT](
             environment=environment,
             service_key=service_key,
             config=config,
@@ -424,70 +435,84 @@ def create(
             raise ValueError(
                 "Argument 'client_key' can not be None if 'logger_type' is 'client'"
             )
-        return Client(
+        return Client[EnvironmentT, ServiceKeyT](
             environment=environment,
             service_key=service_key,
             client_key=client_key,
             config=config,
         )
     elif type is LoggerType.CONTROLLER:
-        return Controller(
+        return Controller[EnvironmentT, ServiceKeyT](
             environment=environment,
             service_key=service_key,
             config=config,
         )
     elif type is LoggerType.DATABASE:
-        return Database(
+        return Database[EnvironmentT, ServiceKeyT](
             environment=environment,
             service_key=service_key,
             config=config,
         )
     elif type is LoggerType.EXCEPTION:
-        return Exception(
+        return Exception[EnvironmentT, ServiceKeyT](
             environment=environment,
             service_key=service_key,
             config=config,
         )
     elif type is LoggerType.MIDDLEWARE:
-        return Middleware(
+        return Middleware[EnvironmentT, ServiceKeyT](
             environment=environment,
             service_key=service_key,
             config=config,
         )
     elif type is LoggerType.REPOSITORY:
-        return Repository(
+        return Repository[EnvironmentT, ServiceKeyT](
             environment=environment,
             service_key=service_key,
             config=config,
         )
     elif type is LoggerType.SERVICE:
-        return Service(
+        return Service[EnvironmentT, ServiceKeyT](
             environment=environment,
             service_key=service_key,
             config=config,
         )
 
 
-class ServiceLoggers(BaseModel):
+class ServiceLoggers(BaseModel, Generic[EnvironmentT, ServiceKeyT]):
     model_config = ConfigDict(arbitrary_types_allowed=True)
 
-    application: Application = Field(..., description="Application logger")
-    cache: Cache = Field(..., description="Cache logger")
-    controller: Controller = Field(..., description="Controller logger")
-    database: Database = Field(..., description="Database logger")
-    exception: Exception = Field(..., description="Exception logger")
-    middleware: Middleware = Field(..., description="Middleware logger")
-    repository: Repository = Field(..., description="Repository logger")
-    service: Service = Field(..., description="Service logger")
+    application: Application[EnvironmentT, ServiceKeyT] = Field(
+        ..., description="Application logger"
+    )
+    cache: Cache[EnvironmentT, ServiceKeyT] = Field(..., description="Cache logger")
+    controller: Controller[EnvironmentT, ServiceKeyT] = Field(
+        ..., description="Controller logger"
+    )
+    database: Database[EnvironmentT, ServiceKeyT] = Field(
+        ..., description="Database logger"
+    )
+    exception: Exception[EnvironmentT, ServiceKeyT] = Field(
+        ..., description="Exception logger"
+    )
+    middleware: Middleware[EnvironmentT, ServiceKeyT] = Field(
+        ..., description="Middleware logger"
+    )
+    repository: Repository[EnvironmentT, ServiceKeyT] = Field(
+        ..., description="Repository logger"
+    )
+    service: Service[EnvironmentT, ServiceKeyT] = Field(
+        ..., description="Service logger"
+    )
 
     @classmethod
     def new(
         cls,
         *,
-        environment: Optional[Environment] = None,
-        service_key: Optional[Key] = None,
+        environment: Optional[EnvironmentT] = None,
+        service_key: Optional[ServiceKeyT] = None,
         config: Config,
-    ) -> "ServiceLoggers":
+    ) -> "ServiceLoggers[EnvironmentT, ServiceKeyT]":
         return cls(
             application=create(
                 LoggerType.APPLICATION,
